<?php

/**
 * Do not use or reference this directly from your client-side code.
 * Instead, this should be required via the endpoint.php or endpoint-cors.php
 * file(s).
 */



class UploadHandler {

    public $allowedExtensions = [];
    public $sizeLimit = null;
    public $inputName = 'qqfile';
    public $chunksFolder = 'chunks';

    public $chunksCleanupProbability = 0.001; // Once in 1000 requests on avg
    public $chunksExpireIn = 604800; // One week

    protected $aConfig = [];

    protected $uploadName;

    protected $sBackStep;

    public function __construct(){}

    public function __isset($name)
    {
        return isset($this->aConfig[$name]) || isset($this->$name);
    }

     public function __set($name, $value)
     {
         $this->$name = $value;
         return $this->aConfig[$name] = $value;
     }

    public function __get($name)
    {
        $retval = null;
        if (!$this->__isset($name)) {
            return array('error' => 'SERVER ERROR! Tried to get nonexisting property ['.$name.']');
        }
        $retval = ($this->aConfig[$name] ?: $this->$name);
        return $retval;
    }

/*********************************************************************************************/
/*
*/
/*********************************************************************************************/

    public function set($name, $value = '')
    {
        $this->$name = $value;
        $this->aConfig[$name] = $value;
    }

    public function get($name)
    {
        if (!$this->aConfig[$name] || !$this->$name){return array('error' => 'SERVER ERROR! Tried to get nonexisting property ['.$name.']');}
        return ($this->aConfig[$name]?:$this->$name);
    }

/*********************************************************************************************/
/*
*/
/*********************************************************************************************/
    public function sanitizeFilename($val)
    {
        // whitespace durch Unterstrich ersetzen
    /*
        $sRetval = preg_replace('#(\s+)#', '_', $val);
        $sRetval = preg_replace('/[^A-Za-z0-9]/', '_', $val);
        [<>:"/\\|?*]|            # file system reserved https://en.wikipedia.org/wiki/Filename#Reserved_characters_and_words
    */
        $sRetval = preg_replace(
        '~
        (\s+)|                        # file system reserved
        [\x00-\x1F]|                  # control characters http://msdn.microsoft.com/en-us/library/windows/desktop/aa365247%28v=vs.85%29.aspx
        [\x7F\xA0\xAD]|               # non-printing characters DEL, NO-BREAK SPACE, SOFT HYPHEN
        [#\[\]@!§"$%&\'\?()+,;:=§\/]| # URI reserved https://tools.ietf.org/html/rfc3986#section-2.2
        [{}^\~`]                      # URL unsafe characters https://www.ietf.org/rfc/rfc1738.txt
        ~x',
        '_', $val);
        // Liste aller Umlaute
        $map = array(
                'ä' => 'ae',
                'Ä' => 'ae',
                'ß'=>'ss',
                'ö'=>'oe',
                'Ö' => 'oe',
                'Ü'=>'ue',
                'ü'=>'ue',
                '<'=>'',
                '>'=>'',
                // hier ggf. weitere Zeichen ergänzen, z.B.
                'à' => 'a',
                'é' => 'e',
                'è' => 'e',
            );
//            $sRetval = preg_replace('#^.*?([^/]*?)\.[^\.]*$#i', '\1', $sRetval);
        $sRetval = str_replace('__', '', $sRetval);
        // Umlaute konvertieren
        $sRetval = str_replace(array_keys($map), array_values($map), $sRetval);
        // alle anderen Zeichen verwerfen
    //    $sRetval = preg_replace('#[^a-z0-9_.-]#', '', $sRetval);
        return $sRetval;
    }

    /**
     * Get the original filename
     */
    public function getName(){
        $sRetVal = null;
        if (isset($_REQUEST['qqfilename'])){
            $sRetVal = $_REQUEST['qqfilename'];
        } else if (isset($_FILES[$this->inputName])){
            $sRetVal = $_FILES[$this->inputName]['name'];
        }
        return $this->sanitizeFilename($sRetVal);
    }

    protected function getInitialFiles() {
        $initialFiles = [];
        for ($i = 0; $i < 5000; $i++) {
            array_push($initialFiles, array("name" => "name" + $i, uuid => "uuid" + $i, thumbnailUrl => "/test/dev/handlers/vendor/fineuploader/php-traditional-server/fu.png"));
        }
        return $initialFiles;
    }

    /**
     * Get the name of the uploaded file
     */
    public function getUploadName(){
        return $this->uploadName;
    }

    protected function combineChunks($uploadDirectory, $name = null) {
        $uuid = $_POST['qquuid'];
        if ($name === null){
            $name = $this->getName();
        }
        $targetFolder = $this->chunksFolder.DIRECTORY_SEPARATOR.$uuid;
        $totalParts = isset($_REQUEST['qqtotalparts']) ? (int)$_REQUEST['qqtotalparts'] : 1;
        $targetPath = join(DIRECTORY_SEPARATOR, [$this->sBackStep.$uploadDirectory, $uuid, $name]);

        $this->uploadName = $name;
        if (!file_exists($targetPath)){
            mkdir(dirname($targetPath));
        }
        $target = fopen($targetPath, 'wb');
        for ($i=0; $i<$totalParts; $i++){
            $chunk = fopen($targetFolder.DIRECTORY_SEPARATOR.$i, "rb");
            stream_copy_to_stream($chunk, $target);
            fclose($chunk);
        }
        // Success
        fclose($target);
        for ($i=0; $i<$totalParts; $i++){
            unlink($targetFolder.DIRECTORY_SEPARATOR.$i);
        }
        rmdir($targetFolder);
        if (!is_null($this->sizeLimit) && filesize($targetPath) > $this->sizeLimit) {
            unlink($targetPath);
            http_response_code(413);
            return array("success" => false, "uuid" => $uuid, "preventRetry" => true);
        }
        return array("success" => true, "uuid" => $uuid);
    }

    /**
     * Process the upload.
     * @param string $uploadDirectory Target directory.
     * @param string $name Overwrites the name of the file.
     */
    public function handleUpload($uploadDirectory, $name = null){
        if (is_writable($this->chunksFolder) &&
            1 == mt_rand(1, 1/$this->chunksCleanupProbability)){
            // Run garbage collection
            $this->cleanupChunks();
        }
        // Check that the max upload size specified in class configuration does not
        // exceed size allowed by server config
        if ($this->toBytes(ini_get('post_max_size')) < $this->sizeLimit ||
            $this->toBytes(ini_get('upload_max_filesize')) < $this->sizeLimit){
            $neededRequestSize = max(1, $this->sizeLimit / 1024 / 1024) . 'M';
            return array('error'=>"Server error. Increase post_max_size and upload_max_filesize to ".$neededRequestSize);
        }
        if ($this->isInaccessible($this->sBackStep.$uploadDirectory)){
//            return array('error' =>"Server error. Uploads directory isn't writable or executable. CUR_DIR: " . getcwd() . " DIR: " . ($this->sBackStep));
            return array('error' => 'Server error. Uploads directory '.$uploadDirectory.' isn\'t writable');
        }
        $type = $_SERVER['CONTENT_TYPE'];
        if (isset($_SERVER['HTTP_CONTENT_TYPE'])) {
            $type = $_SERVER['HTTP_CONTENT_TYPE'];
        }
        if(!isset($type)) {
            return array('error' => "No files were uploaded.");
        } else if (strpos(strtolower($type), 'multipart/') !== 0){
            return array('error' => "Server error. Not a multipart request. Please set forceMultipart to default value (true).");
        }
        // Get size and name
        $file = $_FILES[$this->inputName];
        $size = $file['size'];
        if (isset($_REQUEST['qqtotalfilesize'])) {
            $size = $_REQUEST['qqtotalfilesize'];
        }
        if ($name === null){
            $name = $this->getName();
        }
        // check file error
        if($file['error']) {
            return array('error' => 'Upload Error #'.$file['error']);
        }
        // Validate name
        if ($name === null || $name === ''){
            return array('error' => 'File name empty.');
        }
        // Validate file size
        if ($size == 0){
            return array('error' => 'File is empty.');
        }
        if (!is_null($this->sizeLimit) && $size > $this->sizeLimit) {
            return array('error' => 'File is too large.', 'preventRetry' => true);
        }
        // Validate file extension
        $pathinfo = pathinfo($name);
        $ext = isset($pathinfo['extension']) ? $pathinfo['extension'] : '';
        if($this->allowedExtensions && !in_array(strtolower($ext), array_map("strtolower", $this->allowedExtensions))){
            $these = implode(', ', $this->allowedExtensions);
            return array('error' => 'File has an invalid extension, it should be one of '. $these . '.');
        }
        // Save a chunk
        $totalParts = isset($_REQUEST['qqtotalparts']) ? (int)$_REQUEST['qqtotalparts'] : 1;
        $uuid = $_REQUEST['qquuid'];
        if ($totalParts > 1){
        # chunked upload
            $chunksFolder = $this->chunksFolder;
            $partIndex = (int)$_REQUEST['qqpartindex'];
            if (!is_writable($chunksFolder) && !is_executable($this->sBackStep.$uploadDirectory)){
                return array('error' => "Server error. Chunks directory isn't writable or executable.");
            }

            $targetFolder = $this->chunksFolder.DIRECTORY_SEPARATOR.$uuid;
            if (!file_exists($targetFolder)){
                mkdir($targetFolder);
            }

            $targetPath = $targetFolder.'/'.$partIndex;
            $success = move_uploaded_file($_FILES[$this->inputName]['tmp_name'], $targetPath);
            return array("success" => true, "uuid" => $uuid);
        }
        else {
        # non-chunked upload
            $targetPath = join(DIRECTORY_SEPARATOR, [$this->sBackStep.$uploadDirectory, $name]);
//            $targetPath = join(DIRECTORY_SEPARATOR, array($this->sBackStep.$uploadDirectory, $uuid, $name));
            if ($targetPath){
                $this->uploadName = basename($targetPath);
                if (!is_dir(dirname($targetPath))){
                    mkdir(dirname($targetPath));
                }
                if (move_uploaded_file($file['tmp_name'], $targetPath)){
                    return array('success'=> true, "uuid" => $uuid);
                }
            }
            return array('error'=> 'Could not save uploaded file.' .
                'The upload was cancelled, or server error encountered');
        }
    }

    /**
     * Process a delete.
     * @param string $uploadDirectory Target directory.
     * @params string $name Overwrites the name of the file.
     *
     */
    public function handleDelete($uploadDirectory, $name=null)
    {
        if ($this->isInaccessible($this->sBackStep.$uploadDirectory)) {
            return ['error' =>"Server error. Uploads directory isn't writable or executable. CUR_DIR: " . getcwd() . " DIR: " . $uploadDirectory];
            return ['error' => 'Server error. Uploads directory '.$uploadDirectory.' isn\'t writable' . ((!$this->isWindows()) ? " or executable." : ".")];
        }
        if ($name === null){
            $name = $this->getName();
        }
        $targetFolder = $this->sBackStep.$uploadDirectory;
        $url = parse_url($_SERVER['REQUEST_URI'], PHP_URL_PATH);

        $tokens = explode('/', $url);
        $uuid = $tokens[count($tokens)-1];
        $targetFileName = join(DIRECTORY_SEPARATOR, array($targetFolder, $name));
//        $targetPath = join(DIRECTORY_SEPARATOR, array($targetFolder, $uuid));
/*
print '<pre  class="mod-pre rounded">function <span>'.__FUNCTION__.'( '.''.' );</span>  filename: <span>'.basename(__FILE__).'</span>  line: '.__LINE__.' -> <br />';
print_r( $targetFileName ); print '</pre>'; flush (); //  ob_flush();;sleep(10); die();
*/
        if (is_writable($targetFileName)){
            unlink($targetFileName);
            return ["success" => true,
                         "uuid" => $uuid,
                         "path" => $uploadDirectory,
                         "name" => $name
            ];
        } else {
            return ["success" => false,
                "error" => "File not found! Unable to delete.".$url,
//                "path"  => $uuid,
                "uuid" => $uuid,
                "path" => $uploadDirectory,
                "name" => $name
            ];
        }
    }

    /**
     * Returns a path to use with this upload. Check that the name does not exist,
     * and appends a suffix otherwise.
     * @param string $uploadDirectory Target directory
     * @param string $filename The name of the file to use.
     */
    protected function getUniqueTargetPath($uploadDirectory, $filename)
    {
        // Allow only one process at the time to get a unique file name, otherwise
        // if multiple people would upload a file with the same name at the same time
        // only the latest would be saved.
        if (function_exists('sem_acquire')){
            $lock = sem_get(ftok(__FILE__, 'u'));
            sem_acquire($lock);
        }
        $pathinfo = pathinfo($filename);
        $base = $pathinfo['filename'];
        $ext = isset($pathinfo['extension']) ? $pathinfo['extension'] : '';
        $ext = $ext == '' ? $ext : '.' . $ext;
        $unique = $base;
        $suffix = 0;
        // Get unique file name for the file, by appending random suffix.
        while (file_exists($this->sBackStep.$uploadDirectory . DIRECTORY_SEPARATOR . $unique . $ext)){
            $suffix += rand(1, 999);
            $unique = $base.'-'.$suffix;
        }
        $result =  $uploadDirectory . DIRECTORY_SEPARATOR . $unique . $ext;
        // Create an empty target file
        if (!touch($result)){
            // Failed
            $result = false;
        }
        if (function_exists('sem_acquire')){
            sem_release($lock);
        }
        return $result;
    }

    /**
     * Deletes all file parts in the chunks folder for files uploaded
     * more than chunksExpireIn seconds ago
     */
    protected function cleanupChunks(){
        foreach (scandir($this->chunksFolder) as $item){
            if ($item == "." || $item == ".."){continue;}
            $path = $this->chunksFolder.DIRECTORY_SEPARATOR.$item;
            if (!is_dir($path)){continue;}
            if (time() - filemtime($path) > $this->chunksExpireIn){
                $this->removeDir($path);
            }
        }
    }

    /**
     * Removes a directory and all files contained inside
     * @param string $dir
     */
    protected function removeDir($dir){
        foreach (scandir($dir) as $item){
            if ($item == "." || $item == ".."){continue;}
            if (is_dir($item)){
                $this->removeDir($item);
            } else {
                unlink(join(DIRECTORY_SEPARATOR, array($dir, $item)));
            }
        }
        rmdir($dir);
    }

    /**
     * Converts a given size with units to bytes.
     * @param string $str
     */
    protected function toBytes($str){
        $val = intval(trim($str));
        $last = strtolower($str[strlen($str)-1]);
        switch($last) {
            case 'g': $val *= 1024;
            case 'm': $val *= 1024;
            case 'k': $val *= 1024;
        }
        return $val;
    }

    /**
     * Determines whether a directory can be accessed.
     *
     * is_executable() is not reliable on Windows prior PHP 5.0.0
     *  (http://www.php.net/manual/en/function.is-executable.php)
     * The following tests if the current OS is Windows and if so, merely
     * checks if the folder is writable;
     * otherwise, it checks additionally for executable status (like before).
     *
     * @param string $directory The target directory to test access
     */
    protected function isInaccessible($directory) {
        $isWin = $this->isWindows();
        $folderInaccessible = ($isWin) ? !is_writable($directory) : ( !is_writable($directory) && !is_executable($directory) );
        return $folderInaccessible;
    }

    /**
     * Determines is the OS is Windows or not
     *
     * @return boolean
     */
    protected function isWindows() {
        $isWin = (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN');
        return $isWin;
    }

} // end of class UploadHandler
